package org.dgraves.DialTones;

import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Intent;
import android.database.ContentObserver;
import android.media.AudioManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TwoLineListItem;
import android.widget.AdapterView.OnItemClickListener;

/**
 * DialTonesView is the main activity for the DialTones application.  
 * It provides a list of contacts with phone numbers from which the
 * user can chose to dial. When the user selects a number from the 
 * list, the PhoneDialerView activity is launched. DialTonesView
 * also provides the application's options menu.
 * 
 * @author Dustin Graves
 */
public class DialTonesView extends ListActivity {
	/**
	 * Extends ContentObserver to receive notification of changes
	 * to the database of contacts and update the list activity
	 * so that it displays the most recent information
	 * 
	 * @author Dustin Graves
	 */
	private class PhoneContactObserver extends ContentObserver {
		PhoneContactObserver(Handler handler) {
			super(handler);
		}

		@Override
		public boolean deliverSelfNotifications() {
			return false;
		}

		/**
		 * Populate the DialTonesActivity's ListView with the latest
		 * contact information.
		 */
		@Override
		public void onChange(boolean selfChange) {
			super.onChange(selfChange);
			populateList();
		}
	}

	/**
	 * Called when the activity is first created. Creates the list of
	 * contacts and associates the activity with the DTMF audio stream.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Ensure that the DTMF stream is adjusted by the volume keys
		setVolumeControlStream(AudioManager.STREAM_DTMF);

		setupList();
	}

	/** Create the application's options menu. */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.main_menu, menu);
	    return true;
	}

	/**
	 * Launches the appropriate activity when an option is selected
	 * from the options menu: PhoneDialerPreferencesView activity or
	 * info dialog.
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	    case R.id.preferences:
			Intent intent = new Intent(DialTonesView.this, PhoneDialerPreferencesView.class);
            startActivity(intent);
	        return true;
	    case R.id.info:
	    	showDialog(DIALOG_INFO_ID);
	        return true;
	    default:
	        return super.onOptionsItemSelected(item);
	    }
	}

	/**
	 * Display the soft keyboard when the device's search button is pressed,
	 * for applying filters to the contact list.
	 */
	@Override
	public boolean onSearchRequested() {
		// Show the soft keyboard for input to ListView search filter
		InputMethodManager manager = (InputMethodManager)getSystemService(INPUT_METHOD_SERVICE);
		manager.showSoftInput(getListView(), 0);
		return false;
	}

	@Override
	protected Dialog onCreateDialog(int id) {
	    Dialog dialog = null;
	    switch(id) {
	    case DIALOG_INFO_ID:
	    	return createAboutDialog();
	    }
	    return dialog;
	}	

	/** Create the info dialog. */
	private Dialog createAboutDialog() {
		LayoutInflater inflater = (LayoutInflater)getSystemService(LAYOUT_INFLATER_SERVICE);
		View layout = inflater.inflate(R.layout.info_dialog, (ViewGroup)findViewById(R.id.layout_root));

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setView(layout);
		builder.setIcon(R.drawable.icon);
		builder.setTitle(getString(R.string.app_name) + " " + getString(R.string.app_version));
		return builder.create();
	}

	/**
	 * Populate the ListView with the contact data, register content observer 
	 * to update the ListView when contact data is changed externally, and 
	 * add an item click listener to launch the PhoneDialerView activity when
	 * a list item is clicked.
	 */
	private void setupList() {
		// Fill the list
		populateList();

		// Register observer for notification of changes to contacts
		getContentResolver().registerContentObserver(PhoneContactQuery.makeUri(), true, new PhoneContactObserver(new Handler()));

		// List settings and listeners
		ListView lv = getListView();
		lv.setTextFilterEnabled(true);

		lv.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				// When clicked, emit tones to dial number
				TwoLineListItem item = (TwoLineListItem) view;
				if (item.getText2() != null) {
					CharSequence phoneNumber = item.getText2().getText();
					Intent intent = new Intent(Intent.ACTION_DIAL, Uri.parse("tel:" + phoneNumber), DialTonesView.this, PhoneDialerView.class);
		            startActivity(intent);
				}
			}
		});
	}

	/** Create a PhoneContactArrayAdapter to bind the contact data with the ListActivity. */
	private void populateList() {
		setListAdapter(new PhoneContactArrayAdapter(this, R.layout.phone_list_item, getContacts()));
	}

	/**
	 * Query for contacts with associated phone numbers to display in the ListView. 
	 * Displays an alert dialog if the query fails.
	 * @return A list of PhoneContact objects containing contact name and phone number data.
	 * @see PhoneContact
	 */
	private List<PhoneContact> getContacts() {
		PhoneContactQuery query = new PhoneContactQuery(this);
		try {
			return query.getContacts();
		} catch (Exception e) {
			// Show error
			MessageDialogBuilder.createAlertDialog(this, getString(R.string.contact_fail_title), e.getMessage()).show();

			// Return an empty list
			return new ArrayList<PhoneContact>();
		}
	}

	/** Id for the application's info dialog. */ 
	private static final int DIALOG_INFO_ID = 0;
}
