package org.dgraves.DialTones;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Filter;
import android.widget.TwoLineListItem;

/** ArrayAdapter class to adapt a PhoneContact list to a TwoLineListItem view.
 * Based on tutorial from:
 * http://shri.blog.kraya.co.uk/2010/04/19/android-multi-line-select-list/
 * @author Dustin Graves
 */
public class PhoneContactArrayAdapter extends ArrayAdapter<PhoneContact> {
	/**
	 *  PhoneContactFilter class to apply a custom filter to the PhoneContact
	 *  objects stored by the encapsulated List. Based on ArrayAdapter.ArrayFilter.
	 *  
	 * @author Dustin Graves
	 * @see PhoneContact
	 */
	private class PhoneContactFilter extends Filter {
		/**
		 * Construct a new ArrayList from the List of PhoneContact objects that
		 * only contains PhoneContact objects with values matching the specified
		 * constraint. Values are matched first by checking for an exact match at
		 * the start of the display name and phone number, and second by matching
		 * each word in the display name and phone number with the constraint.
		 * @param constraint The pattern to match with the PhoneContact data.
		 * @return           A FilterResults objects containing a list of PhoneContact
		 *                   objects whose data matched the constraint value.
		 * @see PhoneContact.
		 */
		@Override
		protected FilterResults performFiltering(CharSequence constraint) {
			FilterResults results = new FilterResults();
			if (constraint == null || constraint.length() == 0) {
				results.values = new ArrayList<PhoneContact>(contacts);
				results.count = contacts.size();
			}
			else {
				int contactCount = contacts.size();
				ArrayList<PhoneContact> filteredContacts = new ArrayList<PhoneContact>(contactCount);
				String prefix = constraint.toString().toLowerCase();
				
				for (int i = 0; i < contactCount; i++) {
					PhoneContact contact = contacts.get(i);
					String name = makePhoneContactNameLabel(contact).toLowerCase();
					String number = contact.getNumber().toLowerCase();

					// First check for exact match, then check for substring match if no exact match
					if (name.startsWith(prefix) || number.startsWith(prefix)) {
						filteredContacts.add(contact);
					} else {
						// Search string array composed of all individual words from name and number
						String[] words = (name + " " + number).split("\\W+");
						int wordCount = words.length;
						
						for (int j = 0; j < wordCount; j++) {
							if (words[j].startsWith(prefix)) {
								filteredContacts.add(contact);
								break;
							}
						}
					}
				}
				
				results.values = filteredContacts;
				results.count = filteredContacts.size();
			}
			
			return results;
		}

		@SuppressWarnings("unchecked")
		@Override
		protected void publishResults(CharSequence constraint, FilterResults results) {
			// Clear existing items from ArrayAdapter to prepare list for population with results
			clear();

			if (results.count > 0) {
				// Populate ArrayAdapter with results
				List<PhoneContact> contacts = (List<PhoneContact>)results.values;
				int contactCount = contacts.size();
				for (int i = 0; i < contactCount; i++) {
					add(contacts.get(i));
				}
				
				notifyDataSetChanged();
			} else {
				notifyDataSetInvalidated();
			}
		}		
	}

	/**
	 * Construct the PhoneContactArrayAdapter object to generate TwoLineListItem objects
	 * containing contact display name and phone number for display in the DialTonesView
	 * object's ListView.
	 * @param context Context object for the ArrayAdapter.
	 * @param textViewResourceId Resource id for the TwoLineListItem to display contact display name and phone number. 
	 * @param objects List of PhoneContact objects to display.
	 * @see PhoneContact.
	 */
	public PhoneContactArrayAdapter(Context context, int textViewResourceId, List<PhoneContact> objects) {
		super(context, textViewResourceId, objects);
		this.contacts = new ArrayList<PhoneContact>(objects);
		this.resourceId = textViewResourceId;
	}
	
	/**
	 * Create a TwoLineListItem to display in the contact list. The first line contains
	 * the contact name, formatted as 'display name (phone number label)' and the second
	 * line contains the phone number to dial. The phone number label is only included
	 * with the first line if it contains a non-empty value.
	 */
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		PhoneContact contact = getItem(position);
		if (contact != null) {
			TwoLineListItem view;
			if (convertView == null) {
				// Instantiate View object from layout XML file
				LayoutInflater inflater = (LayoutInflater)getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				view = (TwoLineListItem)inflater.inflate(resourceId, parent, false);
			} else {
				view = (TwoLineListItem)convertView;
			}

			if (view.getText1() != null) {
				view.getText1().setText(makePhoneContactNameLabel(contact));
			}

			if (view.getText2() != null) {
				view.getText2().setText(contact.getNumber());
			}
			
			return view;
		}
		
		return null;
	}

	@Override
	public Filter getFilter() {
		if (filter == null) {
			filter = new PhoneContactFilter();
		}
		
		return filter;
	}

	/**
	 * Creates a label for the specified PhoneContact object with the
	 * form 'display name (phone number label)'. The phone number label is only included
	 * if it contains a non-empty value.
	 * @param contact PhoneContact object with the label and display name to use for creating a display name for the contact list.
	 * @return String containing the display name to show in the contact list.
	 */
	private String makePhoneContactNameLabel(PhoneContact contact) {
		String label = contact.getLabel();
		if (label == null) {
			return contact.getDisplayName();
		} else {
			return contact.getDisplayName() + " (" + label + ")";
		}
	}

	/** Filter object for "searching" the list of contacts. */
	private PhoneContactFilter filter;
	/** List of all contacts, to be used when filtering contacts to display. */
	final private List<PhoneContact> contacts;
	/** Resource id for the LayoutInflater used to create the contact list's TwoLineListItem. */
	private int resourceId;
}
