package org.dgraves.DialTones;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.database.Cursor;
import android.net.Uri;
import android.provider.ContactsContract.CommonDataKinds.Phone;

/**
 * Class to perform a query for all contacts associated with phone numbers, extracting
 * the contact's display name, phone number, and phone number type. Contact data is 
 * stored in a PhoneContact object. More than one PhoneContact object will be created
 * for contacts with multiple associated phone numbers, with one object per phone number.
 * @author Dustin Graves
 */
public class PhoneContactQuery {
	/**
	 * Create the query object for extracting contact display name, phone number, and 
	 * phone number type. 
	 * @param activity Activity to use for performing the query and extracting the phone number type label.
	 */
	PhoneContactQuery(Activity activity) {
		this.activity = activity;
	}

	/** 
	 * Perform an unfiltered query for contact data.
	 * @return list of all PhoneContact objects for all contacts with associated phone numbers.
	 */
	public List<PhoneContact> getContacts() {
		return getContacts(null);
	}

	/**
	 * Perform a filtered query for contact data.
	 * @param filter Filter used by {@link #contactQuery(String)} when creating the URI for the contact query.
	 * @return list of PhoneContact objects for contacts with associated phone numbers that matched the filter. 
	 */
	public List<PhoneContact> getContacts(String filter) {
		List<PhoneContact> contacts = new ArrayList<PhoneContact>();
		Cursor cursor = contactQuery(filter);
		if (cursor.moveToFirst()) {
			int displayNameIndex = cursor.getColumnIndex(Phone.DISPLAY_NAME);
			int numberIndex = cursor.getColumnIndex(Phone.NUMBER);
			int typeIndex = cursor.getColumnIndex(Phone.TYPE);
			int labelIndex = cursor.getColumnIndex(Phone.LABEL);
			
			do {
				// Create PhoneContact object, obtaining the descriptive string associated
				// with the Phone.TYPE index associated with the contact
				PhoneContact contact = new PhoneContact(cursor.getString(displayNameIndex),
						cursor.getString(numberIndex),
						Phone.getTypeLabel(activity.getResources(),
			                    cursor.getInt(typeIndex),
			                    cursor.getString(labelIndex)).toString());
				contacts.add(contact);
			} while (cursor.moveToNext());
		}
		
		return contacts;
	}

	/**
	 * Create a phone content URI without a filter.
	 * @return URI to use when querying for contacts with associated phone numbers.
	 */
	public static final Uri makeUri() {
		return makeUri(null);
	}

	/**
	 * Create a phone content URI with a filter.
	 * @return URI to use when querying for contacts with associated phone numbers.
	 */
	public static final Uri makeUri(String filter) {
		if (filter == null || filter == "") {
			return Phone.CONTENT_URI;
		} else {
			return Uri.withAppendedPath(Phone.CONTENT_FILTER_URI, Uri.encode(filter));
		}
	}

	/**
	 * Perform query for contacts with associated phone numbers.
	 * @param filter Filter used to create URI for the contact query.
	 * @return Cursor providing access to query results.
	 */
	private Cursor contactQuery(String filter) {
		return activity.managedQuery(PhoneContactQuery.makeUri(filter),
				new String[] { Phone.DISPLAY_NAME, Phone.NUMBER, Phone.TYPE, Phone.LABEL },
				null,
				null,
				Phone.DISPLAY_NAME + " ASC");
	}
	
	/** Activity to use for performing the query and retrieving the phone number type label. */
	private Activity activity;
}
