package org.dgraves.DialTones;

import android.media.AudioManager;
import android.media.ToneGenerator;

/**
 * Generates DTMF signals for a specified phone number.
 * @author Dustin Graves
 */
public class PhoneDialer {
	// Minimum DTMF tone durations as described here:
	//	http://nemesis.lonestar.org/reference/telecom/signaling/dtmf.html
	/** Minimum mark (sound) duration in milliseconds. */
	public static final int DTMF_MINIMUM_MARK = 45;
	/** Minimum space (silence) duration in milliseconds. */
	public static final int DTMF_MINIMUM_SPACE = 40;
	/** Minimum duration required by RadioShack in milliseconds. */
	public static final int DTMF_MINIMUM_RS = 70;

	/**
	 * Create a PhoneDialer object with custom durations and volume.
	 * @param markDuration Period of time for which the signal should be emitted.
	 * @param spaceDuration Period of silence between signals.
	 * @param volume Volume level to be specified when creating the ToneGenerator object.
	 */
	PhoneDialer(int markDuration, int spaceDuration, int volume) {
		this.markDuration = markDuration;
		this.spaceDuration = spaceDuration;
		this.volume = volume;
	}

	/**
	 * Create a PhoneDialer object with custom volume.
	 * @param volume Volume level to be specified when creating the ToneGenerator object.
	 */
	PhoneDialer(int volume) {
		this(DTMF_MINIMUM_RS, DTMF_MINIMUM_RS, volume);
	}
	
	/**
	 * Create a PhoneDialer object with default durations and volume.
	 */
	PhoneDialer() {
		this(DTMF_MINIMUM_RS, DTMF_MINIMUM_RS, ToneGenerator.MAX_VOLUME);
	}

	public int getVolume() {
		return volume;
	}
	
	public int getMarkDuration() {
		return markDuration;
	}
	
	public int getSpaceDuration() {
		return spaceDuration;
	}

	public void setVolume(int volume) {
		this.volume = volume;
	}
	
	public void setMarkDuration(int markDuration) {
		this.markDuration = markDuration;
	}
	
	public void setSpaceDuration(int spaceDuration) {
		this.spaceDuration = spaceDuration;
	}
	
	/** Function to generate a sequence of DTMF tones for the 
	 *  specified phone number.  Recognizes values '0'-'9', '#', '*', 
	 *  and ','.  All other values are ignored.
	 *  @param phoneNumber - String containing the phone number to dial.
	 */
	public void dial(CharSequence phoneNumber) throws InterruptedException {
		ToneGenerator generator = new ToneGenerator(AudioManager.STREAM_DTMF, volume);
		for (int i = 0; i < phoneNumber.length(); ++i) {
			char value = phoneNumber.charAt(i);
			if (Character.isDigit(value)) {
				playTone(generator, tones[Character.digit(value, 10)]);
			} else if(value == '#') {
				playTone(generator, ToneGenerator.TONE_DTMF_P);
			} else if(value == '*') {
				playTone(generator, ToneGenerator.TONE_DTMF_S);
			} else if(value == ',') {
				// Pause for 1 second
				Thread.sleep(1000);
			}
		}
		
		generator.release();
	}

	/**
	 * Emit a single tone for the specified mark duration, followed by silence for the 
	 * specified space duration. 
	 * @param generator ToneGenerator to emit the tone.
	 * @param tone Tone to generate.
	 * @throws InterruptedException Exception thrown if thread is interrupted while sleeping for a duration.
	 */
	private void playTone(ToneGenerator generator, int tone) throws InterruptedException {
		// DTMF Mark
		try {
			generator.startTone(tone);
			Thread.sleep(markDuration);
		} finally {
			generator.stopTone();
		}
		
		//DTMF Space
		Thread.sleep(spaceDuration);
	}

	/** Map of phone number digits to DTMF tones. */
	private static final int[] tones = new int[] {
            ToneGenerator.TONE_DTMF_0,
            ToneGenerator.TONE_DTMF_1,
            ToneGenerator.TONE_DTMF_2,
            ToneGenerator.TONE_DTMF_3,
            ToneGenerator.TONE_DTMF_4,
            ToneGenerator.TONE_DTMF_5,
            ToneGenerator.TONE_DTMF_6,
            ToneGenerator.TONE_DTMF_7,
            ToneGenerator.TONE_DTMF_8,
            ToneGenerator.TONE_DTMF_9
	};

	/** Duration for DTMF tone generation in milliseconds. */
	private int markDuration;

	/** Duration for silence between DTMF tones in milliseconds. */
	private int spaceDuration;
	
	/** Volume level for DTMF audio stream. */
	private int volume;
}
